
/******************************************************************************
*	SDIB32GL Factory Class Source
******************************************************************************/
#include "stdafx.h"
#include "sdib32gl.h"
#include "sdib32glbase.h"
#include "sdib32glx86.h"
#include "sdib32glpmmx.h"
#include "sdib32glfactory.h"

#include "_sdib32glfactory.cpp"


/*----------------------------------------------------------------------------*
|	RXgN^
*----------------------------------------------------------------------------*/
CSDib32GLFactory::CSDib32GLFactory()
{
	m_dwUseSysMem = 0;

	CDebugOut::Put( "\t@new ISDib32GLFactory : adr=0x%08X;\n", this );
}


/*----------------------------------------------------------------------------*
|	fXgN^
*----------------------------------------------------------------------------*/
CSDib32GLFactory::~CSDib32GLFactory()
{
	list<CSDib32GLBase*>::iterator it = m_SDib32GLList.begin();

	while( it != m_SDib32GLList.end() )
	{
		// o^ĂIuWFNg
		//
		// œo^邱ƂɂăC^[tFCX
		// ̏̋K𖳂
		(*it++)->Unregister();
	}

	CDebugOut::Put( "\t@del ISDib32GLFactory : adr=0x%08X;\n", this );
}


/*----------------------------------------------------------------------------*
|	
*----------------------------------------------------------------------------*/
STDMETHODIMP CSDib32GLFactory::Initialize( SDIB32GLCREATEMODE Mode )
{
	if( m_pFactory.get() != NULL ) return SDIB32GLERR_ALREADYINITIALIZED;

	// CPU@\`FbN
	if( Mode == SDIB32GLCREATE_AUTO )
	{
		if( ::SDIB32GLIsMMX() )
		{
			Mode = SDIB32GLCREATE_PMMX;
		}else
		{
			Mode = SDIB32GLCREATE_X86;
		}
	}

	try
	{
		// gpNXt@Ng𐶐
		switch( Mode )
		{
		case SDIB32GLCREATE_X86:

			m_pFactory = auto_ptrEx<CFactoryBase>( new CFactory<CSDib32GLX86> );
			break;
		case SDIB32GLCREATE_PMMX:

			m_pFactory = auto_ptrEx<CFactoryBase>( new CFactory<CSDib32GLPMMX> );
			break;
		default:
			return S_FALSE;
		}
	}
	catch( bad_alloc )
	{
		return SDIB32GLERR_OUTOFMEMORY;
	}

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	SDib32GLCX^X̐
*----------------------------------------------------------------------------*/
INLINEMETHODIMP CSDib32GLFactory::CreateInstance( REFIID riid, void** ppvObj )
{
	if( m_pFactory.get() == NULL ) return SDIB32GLERR_NOTINITIALIZED;

	// CX^X擾
	CSDib32GLBase* pSDib32GLObj = 
		static_cast<CSDib32GLBase*>(m_pFactory->CreateInstance());

	// mێs
	if( pSDib32GLObj == NULL )
	{
		*ppvObj = NULL;
		return E_OUTOFMEMORY;
	}
	
	HRESULT hr = pSDib32GLObj->QueryInterface( riid, ppvObj );

	pSDib32GLObj->Release();

	// C^[tFCX݂͑Ȃ
	if( FAILED(hr) )
	{
		return hr;
	}

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	gp̃VXẽTCY擾
*----------------------------------------------------------------------------*/
STDMETHODIMP CSDib32GLFactory::GetUseSysMem( LPDWORD lpdwUse )
{
	if( lpdwUse == NULL ) return E_POINTER;

	*lpdwUse = m_dwUseSysMem;

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	obt@JEg擾
*----------------------------------------------------------------------------*/
STDMETHODIMP CSDib32GLFactory::GetBufferCount( LPDWORD lpdwCount )
{
	if( lpdwCount == NULL ) return E_POINTER;

	*lpdwCount = m_SDib32GLList.size();

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	SDIB32GLobt@̗
*----------------------------------------------------------------------------*/
STDMETHODIMP CSDib32GLFactory::EnumBuffers( LPSDIB32GLENUMBUFFERSCALLBACK lpEnumBuffersCallback, LPARAM lParam )
{
	if( lpEnumBuffersCallback == NULL ) return E_POINTER;

	list<CSDib32GLBase*>::iterator it = m_SDib32GLList.begin();

	while( it != m_SDib32GLList.end() )
	{
		// 񋓒ɃC^[tFCX̉̂ꂽ̂Ƃl
		// ̎̃m[h炩ߎ擾Ă
		//
		// 悭oǑƂȂ鎞̃p^[
		// (ԈႢ~)
		// lpEnumBuffersCallback( *tmp, lParam )
		// tmp++;
		// ( delete ̃m[h; ̃m[h->next )
		//
		// ()
		// list<CSDib32GLBase*>::iterator tmp = it;
		// it++;
		// lpEnumBuffersCallback( *tmp, lParam )
		if( lpEnumBuffersCallback( (*it++), lParam ) == FALSE ) break;
	}

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	SDib32GLobt@̍쐬
*----------------------------------------------------------------------------*/
INLINEMETHODIMP CSDib32GLFactory::InternalCreateBuffer( long nWidth, long nHeight, SDIB32GLCOLOR FillColor, HDC hDC, REFIID riid, void** ppvObj )
{
	HRESULT hr = CreateInstance( riid, ppvObj );

	if( FAILED(hr) ) return hr;

	CSDib32GLBase* pSDib32GLObj = static_cast<CSDib32GLBase*>(*ppvObj);

	try
	{
		// obt@쐬
		hr = pSDib32GLObj->CreateBuffer( nWidth, nHeight, FillColor, hDC );

		if( FAILED(hr) ) throw ExceptionEx();

		// t@Ng̓o^
		hr = pSDib32GLObj->Register( this );

		if( FAILED(hr) ) throw ExceptionEx();

	}
	catch( ExceptionEx )
	{
		pSDib32GLObj->Release();
		*ppvObj = NULL;
		return hr;
	}

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	SDib32GLobt@t@C쐬
*----------------------------------------------------------------------------*/
INLINEMETHODIMP CSDib32GLFactory::InternalCreateBufferFromFile( const BSTR bstrFileName, long nOffset, const BSTR bstrSpiDir, SDIB32GLCOLOR ColorKey, HDC hDC, REFIID riid, void** ppvObj )
{
	if( bstrFileName == NULL ) return SDIB32GLERR_INVALIDPARAMS;

	HRESULT hr = CreateInstance( riid, ppvObj );

	if( FAILED(hr) ) return hr;

	CSDib32GLBase* pSDib32GLObj = static_cast<CSDib32GLBase*>(*ppvObj);

	// BSTRANSI
	char szFileName[MAX_PATH];
	char szSpiDir[MAX_PATH];
	const char* pszSpiDir = NULL;

	::wcstombs( szFileName, bstrFileName, MAX_PATH );
	::wcstombs( szSpiDir, bstrSpiDir ? bstrSpiDir : OLESTR(""), MAX_PATH );

	// SPIfBNg󕶎łȂꍇfBNgpXݒ肷
	if( ::strcmp( szSpiDir, "" ) != 0 ) pszSpiDir = szSpiDir;

	try
	{
		// t@Cǂݍ
		hr = pSDib32GLObj->LoadFromFile( szFileName, nOffset, pszSpiDir, ColorKey, hDC );

		if( FAILED(hr) ) throw ExceptionEx();

		// t@Ng̓o^
		hr = pSDib32GLObj->Register( this );

		if( FAILED(hr) ) throw ExceptionEx();

	}
	catch( ExceptionEx )
	{
		pSDib32GLObj->Release();
		*ppvObj = NULL;
		return hr;
	}

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	SDib32GLobt@쐬
*----------------------------------------------------------------------------*/
INLINEMETHODIMP CSDib32GLFactory::InternalCreateBufferFromMemory( const void* pvMemory, long nSize, const BSTR bstrSpiDir, SDIB32GLCOLOR ColorKey, HDC hDC, REFIID riid, void** ppvObj )
{
	if( pvMemory == NULL )	return E_POINTER;

	HRESULT hr = CreateInstance( riid, ppvObj );

	if( FAILED(hr) ) return hr;

	CSDib32GLBase* pSDib32GLObj = static_cast<CSDib32GLBase*>(*ppvObj);

	// BSTRANSI
	char szSpiDir[MAX_PATH];
	const char* pszSpiDir = NULL;
	::wcstombs( szSpiDir, bstrSpiDir ? bstrSpiDir : OLESTR(""), MAX_PATH );

	// SPIfBNg󕶎łȂꍇfBNgpXݒ肷
	if( ::strcmp( szSpiDir, "" ) != 0 ) pszSpiDir = szSpiDir;

	try
	{
		// ǂݍ
		hr = pSDib32GLObj->LoadFromMemory( pvMemory, nSize, pszSpiDir, ColorKey, hDC );

		if( FAILED(hr) ) throw ExceptionEx();

		// t@Ng̓o^
		hr = pSDib32GLObj->Register( this );

		if( FAILED(hr) ) throw ExceptionEx();

	}
	catch( ExceptionEx )
	{
		pSDib32GLObj->Release();
		*ppvObj = NULL;
		return hr;
	}

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	SDib32GLobt@\[X쐬
*----------------------------------------------------------------------------*/
INLINEMETHODIMP CSDib32GLFactory::InternalCreateBufferFromResource( HINSTANCE hInstance, const BSTR bstrBitmapID, SDIB32GLCOLOR ColorKey, HDC hDC, REFIID riid, void** ppvObj )
{
	if( bstrBitmapID == NULL ) return SDIB32GLERR_INVALIDPARAMS;

	HRESULT hr = CreateInstance( riid, ppvObj );

	if( FAILED(hr) ) return hr;

	CSDib32GLBase* pSDib32GLObj = static_cast<CSDib32GLBase*>(*ppvObj);

	char szBitmapID[MAX_PATH];
	const char* pszBitmapID = szBitmapID;

	if( (long)bstrBitmapID & 0xFFFF0000 )
	{
		// ID
		::wcstombs( szBitmapID, bstrBitmapID, MAX_PATH );
	}else
	{
		// ID
		pszBitmapID = (const char*)bstrBitmapID;
	}

	try
	{
		// \[Xǂݍ
		hr = pSDib32GLObj->LoadFromResource( hInstance, pszBitmapID, ColorKey, hDC );

		if( FAILED(hr) ) throw ExceptionEx();

		// t@Ng̓o^
		hr = pSDib32GLObj->Register( this );

		if( FAILED(hr) ) throw ExceptionEx();

	}
	catch( ExceptionEx )
	{
		pSDib32GLObj->Release();
		*ppvObj = NULL;
		return hr;
	}

	return S_OK;
}


/*----------------------------------------------------------------------------*
|	SDib32GLobt@𕡐
*----------------------------------------------------------------------------*/
INLINEMETHODIMP CSDib32GLFactory::InternalDuplicateBuffer( CSDib32GLBase* pOriginal, REFIID riid, void** ppvObj )
{
	if( pOriginal == NULL ) return SDIB32GLERR_INVALIDOBJECT;

	HRESULT hr = CreateInstance( riid, ppvObj );

	if( FAILED(hr) ) return hr;

	CSDib32GLBase* pSDib32GLObj = static_cast<CSDib32GLBase*>(*ppvObj);

	try
	{
		// obt@Rs[
		hr = pSDib32GLObj->CopyBuffer( pOriginal );

		if( FAILED(hr) ) throw ExceptionEx();

		// t@Ng̓o^
		hr = pSDib32GLObj->Register( this );

		if( FAILED(hr) ) throw ExceptionEx();

	}
	catch( ExceptionEx )
	{
		pSDib32GLObj->Release();
		*ppvObj = NULL;
		return hr;
	}
	
	return S_OK;
}
